@@ -74,8 +74,7 @@ int vio_errno(Vio *vio __attribute__((unused)))
 {
   /* These transport types are not Winsock based. */
 #ifdef _WIN32
-  if (vio->type == VIO_TYPE_NAMEDPIPE ||
-      vio->type == VIO_TYPE_SHARED_MEMORY)
+  if (vio->type == VIO_TYPE_NAMEDPIPE)
     return GetLastError();
 #endif
 
@@ -149,6 +148,10 @@ int vio_socket_io_wait(Vio *vio, enum enum_vio_io_event event)
 #define VIO_DONTWAIT 0
 #endif
 
+#ifndef SOCKET_EAGAIN
+#define SOCKET_EAGAIN SOCKET_EWOULDBLOCK
+#endif
+
 /*
   returns number of bytes read or -1 in case of an error
 */
@@ -363,7 +366,7 @@ int vio_blocking(Vio *vio, my_bool set_blocking_mode, my_bool *old_mode)
   r= set_blocking_mode ? 0 : 1;
 #endif /* !defined(NO_FCNTL_NONBLOCK) */
 #else /* !defined(__WIN__) */
-  if (vio->type != VIO_TYPE_NAMEDPIPE && vio->type != VIO_TYPE_SHARED_MEMORY)
+  if (vio->type != VIO_TYPE_NAMEDPIPE)
   { 
     ulong arg;
     int old_fcntl=vio->fcntl_mode;
@@ -435,13 +438,39 @@ int vio_socket_timeout(Vio *vio,
   DBUG_RETURN(ret);
 }
 
+/* Set TCP_NODELAY (disable Nagle's algorithm */
+int vio_nodelay(Vio *vio, my_bool on)
+{
+  int r;
+  int no_delay= MY_TEST(on);
+  DBUG_ENTER("vio_nodelay");
 
-int vio_fastsend(Vio * vio __attribute__((unused)))
+  if (vio->type == VIO_TYPE_NAMEDPIPE || vio->type == VIO_TYPE_SOCKET)
+  {
+    DBUG_RETURN(0);
+  }
+
+  r = mysql_socket_setsockopt(vio->mysql_socket, IPPROTO_TCP, TCP_NODELAY,
+    IF_WIN((const char*), (void*)) &no_delay,
+    sizeof(no_delay));
+
+  if (r)
+  {
+    DBUG_PRINT("warning",
+     ("Couldn't set socket option for fast send, error %d",
+      socket_errno));
+     r = -1;
+  }
+  DBUG_PRINT("exit", ("%d", r));
+  DBUG_RETURN(r);
+}
+
+int vio_fastsend(Vio * vio)
 {
   int r=0;
   DBUG_ENTER("vio_fastsend");
 
-  if (vio->type == VIO_TYPE_NAMEDPIPE ||vio->type == VIO_TYPE_SHARED_MEMORY)
+  if (vio->type == VIO_TYPE_NAMEDPIPE)
   {
     DBUG_RETURN(0);
   }
@@ -454,18 +483,7 @@ int vio_fastsend(Vio * vio __attribute__((unused)))
   }
 #endif                                    /* IPTOS_THROUGHPUT */
   if (!r)
-  {
-#ifdef __WIN__
-    BOOL nodelay= 1;
-#else
-    int nodelay = 1;
-#endif
-
-    r= mysql_socket_setsockopt(vio->mysql_socket, IPPROTO_TCP, TCP_NODELAY,
-                  IF_WIN((const char*), (void*)) &nodelay,
-                  sizeof(nodelay));
-
-  }
+    r = vio_nodelay(vio, TRUE);
   if (r)
   {
     DBUG_PRINT("warning",
@@ -486,7 +504,7 @@ int vio_keepalive(Vio* vio, my_bool set_keep_alive)
                        (int)mysql_socket_getfd(vio->mysql_socket),
                        (int)set_keep_alive));
 
-  if (vio->type != VIO_TYPE_NAMEDPIPE && vio->type != VIO_TYPE_SHARED_MEMORY)
+  if (vio->type != VIO_TYPE_NAMEDPIPE)
   {
     if (set_keep_alive)
       opt = 1;
@@ -505,7 +523,7 @@ int vio_set_keepalive_options(Vio* vio, const struct vio_keepalive_opts *opts)
   struct tcp_keepalive s;
   DWORD  nbytes;
 
-  if (vio->type == VIO_TYPE_NAMEDPIPE || vio->type == VIO_TYPE_SHARED_MEMORY)
+  if (vio->type == VIO_TYPE_NAMEDPIPE)
     return 0;
 
   if (!opts->idle && !opts->interval)
